@@ -9,7 +9,9 @@ import os
 import sysconfig
 import uuid
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Sequence, Tuple
+from typing import Callable, Dict, List, Sequence, Tuple
+
+from .general import _print
 
 import numpy as np
 
@@ -119,6 +121,9 @@ class ContextCpu(XContext):
 
     """
 
+    _cffi_verbose = False
+    _compile_kernels_info = True
+
     @property
     def nplike_array_type(self):
         return np.ndarray
@@ -254,7 +259,7 @@ class ContextCpu(XContext):
         extra_classes=(),
         extra_headers=(),
         compile=True,  # noqa
-    ) -> dict[str, "KernelCpu"]:
+    ) -> Dict[Tuple[str, tuple], "KernelCpu"]:
         # Determine names and paths
         clean_up_so = not module_name
         module_name = module_name or str(uuid.uuid4().hex)
@@ -314,31 +319,28 @@ class ContextCpu(XContext):
             # Only kernel information, but no possibility to call the kernel
             out_kernels = {}
             for pyname, kernel in kernel_descriptions.items():
-                out_kernels[pyname] = KernelCpu(
+                classes = tuple(kernel.description.get_overridable_classes())
+                out_kernels[(pyname, classes)] = KernelCpu(
                     function=None,
                     description=kernel,
                     ffi_interface=None,
                     context=self,
                 )
 
-        for pyname in kernel_descriptions.keys():
-            out_kernels[pyname].source = source
-            out_kernels[pyname].specialized_source = specialized_source
+        for (pyname, _), kernel in out_kernels.items():
+            kernel.source = source
+            kernel.specialized_source = specialized_source
             # TODO: find better implementation?
-            out_kernels[pyname].description.pyname = pyname
+            kernel.description.pyname = pyname
 
-        kernels_with_classes = {
-            (name, tuple(kernel.description.get_overridable_classes())): kernel
-            for name, kernel in out_kernels.items()
-        }
-        return kernels_with_classes
+        return out_kernels
 
     def kernels_from_file(
         self,
         module_name: str,
         kernel_descriptions: Dict[str, Kernel],
         containing_dir=".",
-    ) -> Dict[str, "KernelCpu"]:
+    ) -> Dict[Tuple[str, tuple], "KernelCpu"]:
         """
         Import a compiled module `module_name` located in `containing_dir`
         (by default it is the current working directory), and add the kernels
@@ -351,7 +353,8 @@ class ContextCpu(XContext):
         )
         out_kernels = {}
         for pyname, kernel_desc in kernel_descriptions.items():
-            out_kernels[pyname] = KernelCpu(
+            classes = tuple(kernel_desc.get_overridable_classes())
+            out_kernels[(pyname, classes)] = KernelCpu(
                 function=getattr(module.lib, kernel_desc.c_name),
                 description=kernel_desc,
                 ffi_interface=module.ffi,
@@ -372,6 +375,9 @@ class ContextCpu(XContext):
         ffi_interface = cffi.FFI()
         ffi_interface.cdef(cdefs)
 
+        if self._compile_kernels_info:
+            _print("Compiling ContextCpu kernels...")
+
         for pyname, kernel in kernel_descriptions.items():
             if pyname not in cdefs:  # check if kernel not already declared
                 signature = cdef_from_kernel(kernel, pyname)
@@ -407,7 +413,11 @@ class ContextCpu(XContext):
             so_file = str(
                 _so_for_module_name(module_name, containing_dir).absolute()
             )
-            output_file = ffi_interface.compile(target=so_file, verbose=True)
+            output_file = ffi_interface.compile(
+                target=so_file, verbose=self._cffi_verbose
+            )
+            if self._compile_kernels_info:
+                _print("Done compiling ContextCpu kernels.")
             return Path(output_file)
         finally:
             # Clean temp files
